*-- File: DATEEX.CC

CLASS DateEx  && extended date
*----------------------------------------------------------------------
*-- Description:
*
*      This class contains many useful date-handling methods. For 
*      'Holidays' and 'Hebrew dates', check the files: "HOLIDAY.CC" 
*      and "HEBREW.CC"
*
*-- Programmers: 
*
*      Ken Mayer
*      Jay Parsons (CIS: 72662,1302)
*      Martin Leon
*      Kenneth Chan 
*      David Love (CIS: 70153,2433)
*      Miriam Liskin
*      Angus Scott-Fleming (76711,517)
*      Todd Barry
*      Romain Strieff (CIS: 73612,2257)
*      Raymond S. Leventhal (CIS: 71544,2140)
*      Bowen Moursund (CIS: 72662,436)
*      David Kanter (CIS: 73252, 662)
*
*-- History: 
*
*      12/23/1996 -- original version of these routines as a class.
*
*-- Instantiation: 
*
*      set procedure to dateex.cc additive
*      _app.Date = new DateEx()
*
*-- Methods:
*      More details on each of these is contained in the commentary for
*      each method ...
*
*--    Release()
*        Releases this object from memory
*
*--    AddMonths()
*        Add months to a date
*
*--    AddYears()
*        Add years to a date
*
*--    Age()
*        Gives 'years' between date passed as parm and date()
*
*--    Age2()
*        'Years' between two dates 
*
*--    Annivrsry()
*        Is a date within a range of dates?
*
*--    Dat2Jul()
*        Convert date to number of days since January 1, 4713 B.C.
*
*--    DateText()
*        Returns date as January 1, 1997
*
*--    DateText2()
*        Returns date as Wednesday, January 1, 1997
*
*--    DateText3()
*        Returns date as January, 1997
*
*--    DoY()
*        Number of days from beginning of year
*
*--    Epoch()
*        Used to confine a date to a 100 year time-span
*
*--    FDoD()
*        First Date of Day of month
*
*--    FDoM()
*        First Day of Month
*
*--    FDoY()
*        First Day of Year
*
*--    FDoNextMonth()
*        First Day of Next Month
*
*--    Fiscal()
*        Returns fiscal year for a given date (based on US Gov't
*        Fiscal year starting in October).
*
*--    IntlCDOW()
*        International Character Day of Week
*
*--    IntlCMON()
*        International Character Month
*
*--    IsLeap()
*        Is the year a leap year?
*
*--    Jul2Dat()
*        Converts 'Julian' date to dBASE date
*
*--    LDoD()
*        Last day of week in month (i.e., last monday of the month)
*
*--    LDoM()
*        Last day of month
*
*--    LDoY()
*        Last day of year
*
*--    NDoW()
*        Numeric Day of Week
*
*--    Num2Dat()
*        Numeric date value to dBASE Date
*
*--    NumDoD()
*        x day of week in a month (i.e., the third monday ...)
*
*--    Quarter()
*        Quarter of the year for specified date
*
*--    StoD()
*        String to date (reverse of DtoS())
*
*--    Text2Date()
*        Converts text date in format: dd-mmm-yy  to dBASE date
*
*--    WeekNo()
*        Week a date falls in from first of year.
*
*-- Properties:
*
*      None
*----------------------------------------------------------------------

   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: dateex.Release()
   *-- Example.....: _app.Date.Release()
   *--               _app.Date = NULL   && not required, but a good 
   *--                                  && idea
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------

      release object this

   RETURN NULL
   *-- EoM: Release()

   FUNCTION AddMonths( dDate, nMonths )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 06/07/1993
   *-- Notes.......: Finds same day as given date N months ahead.
   *--               This function will return the first day of the
   *--               following month if there is no equal date in the 
   *--               month otherwise returned and nMonths is positive, 
   *--               or the last day of the month if nMonths is 
   *--               negative.  That is, a call with {01/31/91} 
   *--               (January 31, 1991) and 1 would yield March 1, 
   *--               there being no February 31. Do not use this 
   *--               function successively to find first the date one 
   *--               month ahead, then the date one month beyond that.
   *--               Instead, to find the date two months ahead from 
   *--               the original date, call this function with the 
   *--               original date and nMonths = 2.  Otherwise, in the
   *--               example, you'll get April 1 the second time rather
   *--               than the correct March 31.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/10/1991 -- Original Release
   *--               06/07/1993 -- Lee Hite (CIS: 71213,2475).  added 
   *--               ROUND function in new date computation to prevent 
   *--               decimals being returned in the date value.  This 
   *--               insures that dBase relational operators will work 
   *--               correctly with the result, i.e., 
   *--               AddMonths({1/1/91},1)={2/1/91} will return true, 
   *--               whereas before it would not, even though the 
   *--               calculated date appeared correct when evaluated as
   *--               a character string.
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.AddMonths(<dDate>,<nMonths>)
   *-- Example.....: ? _app.Date.AddMonths({01/01/91},1)
   *-- Returns.....: Date
   *-- Parameters..: dDate   = Date being tested for ...
   *--               dMonths = Number of months "ahead"
   *-------------------------------------------------------------------

      local dNew, dTest, dReturn

      dNew = dDate - day( dDate )+ 15 ;
                + round( 30.436875 * nMonths, 0 ) &&middle of month
      dTest = dNew - day( dNew ) + day( dDate )
      dReturn = iif( month( dTest ) = month( dNew ), ;
                     dTest, dTest - day( dTest ) ;
                     + iif( nMonths > 0, 1, 0 ) )

   RETURN dReturn
   *-- EoM: AddMonths()

   FUNCTION AddYears( dDate, nYears )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 04/22/1994
   *-- Notes.......: Finds same day as given date N years ahead. 
   *--               Using this function dBASE IV will take care of
   *--               converting February 29 to March 1 if moving from a
   *--               leap to a non-leap year.  However, it may not be 
   *--               used backwards (negative value of nYears) since 
   *--               the date a year before February 29, 1992 will be 
   *--               returned as March 1, 1991, not February 28, 1991.  
   *--               If you must move back, either check explicitly for 
   *--               February 29 as the original date or add code as in
   *--               the AddMonths() function to test for the date 
   *--               returned being of a different month than the 
   *--               original and, if it is, to subtract its day().
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/10/1991 - original function.
   *--               11/14/1991 - Ken Mayer - expanded out to make it
   *--               easier to read, and see what's happening.
   *--               04/22/1994 - Jay Parsons - made international
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.AddYears(<dDate>,<nYears>)
   *-- Example.....: ? _app.Date.AddYears({01/01/91},1)
   *-- Returns.....: Date
   *-- Parameters..: dDate  = Date being tested for ...
   *--               dYears = Number of Years "ahead"
   *-------------------------------------------------------------------
   
      local cYear, cMonth, cDay, cDateSet, dReturn
      
      cYear = str( year( dDate ) + nYears )
      cMonth = right( str( month( dDate ) + 100 ), 2 )
      cDay = right( str( day( dDate ) + 100 ), 2 )
      cDateSet = set("DATE")
      set date AMERICAN
      dReturn = ctod( cMonth + "/" + cDay + "/" + cYear )
      set date &cDateSet.
      
   RETURN dReturn
   *-- EoM: AddYears()

   FUNCTION Age( dBirth )
   *-------------------------------------------------------------------
   *-- Programmer..: Martin Leon
   *-- Date........: 10/23/1991
   *-- Notes.......: Returns age of thing as of date() given its 
   *--               birthdate.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 10/23/1991 -- Original
   *--               08/10/1993 -- Ken Mayer, "stole" algorithm from 
   *--                the routine AGE2. It's more efficient.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.Age(<dBirth>)
   *-- Example.....: ? "Joe is "+ltrim(str(_app.Date.Age(dBirth)))+;
   *--                 " today ..."
   *-- Returns.....: Numeric value in years
   *-- Parameters..: dBirth = birthdate of thing to return age of.
   *-------------------------------------------------------------------

   RETURN ( floor( ( val( dtos( date() ) ) ;
                   - val( dtos( dBirth ) ) ) / 10000 ) )
   *-- EoM: Age()

   FUNCTION Age2( d1, d2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Kenneth Chan 
   *-- Date........: 09/25/1994
   *-- Notes.......: Returns number of full years between two dates, 
   *--               which is the age of a person born on the first 
   *--               date as of the second.
   *--               Returns 0 if second date is less than a year after
   *--               first date, -1 if second date is before first date
   *--               by one year or less, and so forth.  Does not check
   *--               for blank dates.
   *-- Written for.: dBASE IV, 1.1 and above
   *-- Rev. History: 10/23/1991 -- Martin Leon ( HMAN ) - original 
   *--                             function
   *--               04/22/1992 -- Jay Parsons - description modified 
   *--                             and parameters changed
   *--               06/20/1993 -- Ken Chan - replaced, better 
   *--                             algorithm
   *--               07/01/1993 -- Jay Parsons - changed to use floor()
   *--                             instead of int() for coordination 
   *--                             with WorkDays()
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.Age2(<d1>,<d2>)
   *-- Example.....: ? "A person born "+dtoc( d1 )+" was " ;
                       + dtoc( _app.Date.Age2( d1, d2 ) ) +;
                       " years old on " + dtoc( d2 )+"."
   *-- Returns.....: Numeric value in years
   *-- Parameters..: d1 = first date, such as date of birth
   *--               d2 = second date, when age is wanted
   *-------------------------------------------------------------------

   RETURN ( floor( ( val( dtos(d2) ) ;
                  - val( dtos(d1) ) ) / 10000 ) )
   *-- EoM: Age2()

   FUNCTION Annivrsry( dTest, dBegin, dEnd )
   *-------------------------------------------------------------------
   *-- Programmer..: David Love (CIS: 70153,2433) and Jay Parsons
   *--               (CIS: 72662,1302)
   *-- Date........: 11/10/1991
   *-- Notes.......: Checks to see if an anniversary date falls within
   *--               a range of dates (handy for mailings for 
   *--               organizations, checking to see if someone's 
   *--               birthday falls within certain dates, etc.)
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/10/1991 - Original Release
   *--               06/30/1992 - Jay Parsons - revised to work with 
   *--                 AGE2
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: dateex.Age2()
   *-- Usage.......: dateex.Annivrsry(<dTest>,<dBegin>,<dEnd>)
   *-- Example.....: if _app.Date.Annivrsry(dBDay,{03/01/91},{03/31/91})
   *--                  *-- do something
   *--               endif
   *-- Returns.....: .T. if a date (dTest), or its anniversary, falls
   *--               within the period beginning at dBegin or ending at
   *--               dEnd, inclusive.  Returns .F. for any other
   *--               result, including invalid ranges or blank dates.
   *-- Parameters..: dTest  = Date being tested for ...
   *--               dBegin = Beginning of range
   *--               dEnd   = End of range
   *-------------------------------------------------------------------

      local nYears

      nYears = 0
      * This "if" test will fail if a date is blank
      if dBegin <= dEnd .AND. dTest <= dEnd
         nYears = CLASS::Age2( dTest, dEnd ) - ;
                  CLASS::Age2( dTest, dBegin - 1 )
      endif

   RETURN nYears # 0
   *-- EoM: Annivrsry()

   FUNCTION Dat2Jul( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 09/25/1994
   *-- Notes.......: Converts dBASE date to Julian # of days (from 
   *--               January 1, 4713 B.C.)  This is the value of the 
   *--               float dBASE uses to store dates, and is used by 
   *--               astronomers.  Do not use for dBASE dates before 
   *--               1582, or without considering time zones if using 
   *--               for astronomy.
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Written for.: dBASE IV
   *-- Rev. History: None
   *-- Calls.......: None
   *-- Usage.......: dateex.Dat2Jul(<dDate>)
   *-- Example.....: ? _app.Date.Dat2Jul(date())
   *-- Returns.....: Numeric
   *-- Parameters..: dDate = Date to convert to Julian ...
   *-------------------------------------------------------------------

   RETURN ( 2415386 + dDate - ctod( "01/01/01" ) )

   *-- EoM: Dat2Jul

   FUNCTION DateText( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Miriam Liskin
   *-- Date........: 09/25/1994
   *-- Notes.......: Display date in format Month day, year (e.g.,
   *--               July 1, 1991)
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 05/23/1991 -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302                                     ,
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: dateex.DateText(<dDate>)
   *-- Example.....: ? _app.Date.DateText(date())
   *-- Returns.....: July 1, 1991
   *-- Parameters..: dDate = date to be converted
   *-------------------------------------------------------------------

   RETURN ( cmonth(dDate) +" "+ltrim(str(day(dDate),2))+", "+;
                        str(year(dDate) ,4) )
   *-- EoM: DateText

   FUNCTION DateText2( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Miriam Liskin
   *-- Date........: 09/25/1994
   *-- Notes.......: Display date in format day-of-week, Month day, 
   *--               year
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 05/23/1991 -- Original
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: DateEx.DateText
   *-- Usage.......: DateEx.DateText2(<dDate>)
   *-- Example.....: ? _app.Date.DateText2(date())
   *-- Returns.....: the date formatted as: Monday, July 1, 1991
   *-- Parameters..: dDate = date to be converted
   *-------------------------------------------------------------------

   RETURN ( cdow(dDate) +  ", " + CLASS::DateText(dDate) )
   *-- EoM: DateText2

   FUNCTION DateText3( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Miriam Liskin
   *-- Date........: 09/25/1994
   *-- Notes.......: Display date in format Month, year
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 05/21/1991 -- original function.
   *--               03/02/1992 -- This one is Douglas P. Saine's 
   *--                (XRED) invention.  In his words: "I just removed 
   *--                the middle part looking for the day. For the 
   *--                things I do, I only need the month and year. 
   *--                (I work for a defense contractor, accuracy of 
   *--                dates has never been of great concern. <G>)"
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.DateText3(<dDate>)
   *-- Example.....: ? _app.Date.DateText3(date())
   *-- Returns.....: July, 1991
   *-- Parameters..: dDate = date to be converted
   *-------------------------------------------------------------------

   RETURN ( cmonth(dDate)+", "+str(year(dDate),4) )
   *-- EoD: DateText3

   FUNCTION DoY( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 07/07/1993
   *-- Notes.......: Returns the day of the year of a date (from 
   *--               beginning of the year).  Known as "Julian" date 
   *--               in government and military contexts.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/10/1991 -- original function.
   *--               11/14/1991 -- Ken Mayer - expanded for readability
   *--               07/07/1993 -- Jay Parsons - revised to use Zak's 
   *--                 FDoY
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: DateEx.FDoY 
   *-- Usage.......: DateEx.DoY(<dDate>)
   *-- Example.....: ? _app.Date.DoY({01/01/91})
   *-- Returns.....: Numeric value of day of year
   *-- Parameters..: dDate  = Date being tested for ...
   *-------------------------------------------------------------------

   RETURN dDate - CLASS::FDoY( dDate ) + 1
   *-- EoM: DoY()

   FUNCTION Epoch( dDate, xReturn, nEpoch )
   *-------------------------------------------------------------------
   *-- Programmer..: Angus Scott-Fleming (76711,517)
   *-- Date........: 11/03/1996
   *-- Notes.......: Adjusts a date to within 100 years after a given
   *--               Epoch. There are four ways to use Epoch:
   *--               1. Epoch(<date>[,.T.])    returns .t., fixes date,
   *--                                         leaves Epoch unchanged
   *--               2. Epoch(<date>,<year>)   returns fixed date, sets
   *--                                         Epoch to <year>
   *--               3. Epoch(<date>,<date>)   returns fixed date, 
   *--                                         leaves Epoch unchanged.
   *--               4. Epoch(<date>,.t.,<yr>) Returns .t., fixes date,
   *--                                         sets Epoch to <year>
   *--
   *--               Creates custom property of the _app object, 
   *--               _app.dEpoch -- this is the beginning of the 
   *--               100 year time-frame.
   *--
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 10/15/1993 -- Original
   *--               11/03/1996 -- Modified by Ken Mayer to use
   *--                _app.dEpoch, rather than creating a public
   *--                memory variable. Other minor changes to make
   *--                it work as part of the date class.
   *-- Calls.......: DateEx.AddYears()
   *-- Usage.......: DateEx.Epoch(<dDate>,<xReturn>[,<nEpoch>])
   *-- Example.....: Assuming a form (.WFM), assign the valid event
   *--               for a date entryfield:
   *--                    valid _app.Date.Epoch(CLASS::value)
   *--               and the required property should be set to true.
   *-- Returns.....: logical
   *-- Parameters..: dDate    = date being adjusted
   *--               xReturn  = optional to control type of return & 
   *--                          epoch if logical, return .T.
   *--                          if numeric, use as base year for dEpoch
   *--                          if date, use as temporary base for 
   *--                          Epoch
   *--               nEpoch   = optional if logical return is needed to 
   *--                          set beginning year for Epoch.
   *-------------------------------------------------------------------

      local dTemp, dDiff
      
      if type("_app.dEpoch") = "U"
         store {01/01/1950} to _app.dEpoch  && enter your default 
                                            && choice here
      endif
      
      do case
         case pcount() = 3 .and. type("nEpoch") = "N"
            store ctod("01/01/"+str(nEpoch,4,0)) to _app.dEpoch
         case type("xReturn") = "N"
            * set both temporary Epoch AND permanent Epoch
            store ctod("01/01/"+str(xReturn,4,0)) to _app.dEpoch
         otherwise
            * leave existing dEpoch alone
      endcase
       
      dTemp = iif(type("xReturn") = "D",xReturn,_app.dEpoch)
      dDiff = dDate - dTemp
      do while (dDiff < 0 .or. dDiff > 36525)
         do case
            case dDate < dTemp
                 dDate = CLASS::AddYears(dDate,100)
            case dDate => CLASS::AddYears(dTemp,100)
                 dDate = CLASS::AddYears(dDate,-100)
         endcase
         dDiff = dDate - dTemp
      enddo

   RETURN iif(type("xReturn")="L",.t.,dDate)
   *-- EoM: Epoch()

   FUNCTION FDoD( dDate, cDay )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 08/02/1993
   *-- Notes.......: First Date of Day of the week. This function gives 
   *--               the date of the first occurrence in the month of 
   *--               the given day of the week.
   *--               It returns a blank date if the day of week is 
   *--               invalid, but is not case sensitive. 
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 02/25/1992 - Original Release
   *--               08/02/1993 - Jay Parsons - revised to use FDoM().
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: DateEx.FDoM()
   *--	             DateEx.NDoW()
   *-- Usage.......: DateEx.FDoD(<dDate>,<cDay>)
   *-- Example.....: ? _app.Date.FDoD(DATE(),"Tuesday")
   *-- Returns.....: dBASE Date
   *-- Parameters..: dDate  -- date to work from ...
   *--               cDay   -- Day of week to look for ...
   *-------------------------------------------------------------------

      local dReturn, nDay
   
      nDay = CLASS::NDoW(cDay)
      dReturn = CLASS::FDoM( dDate )
   
   RETURN dReturn + mod( nDay + 7 - dow( dReturn ), 7 )
   *-- EoM: FDoD()

   FUNCTION FDoM( dArg )
   *-------------------------------------------------------------------
   *-- Programmer..: Kenneth Chan 
   *-- Date........: 01/05/1993
   *-- Notes.......: First Day of Month 
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 01/05/1993 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.FDoM(<dArg>)
   *-- Example.....: ? _app.Date.FDoM(date())
   *-- Returns.....: Date
   *-- Parameters..: dArg = a Date argument -- function returns first 
   *--                      day of the month of this date.
   *-------------------------------------------------------------------

   RETURN ( ( dArg ) - day( dArg ) + 1 )
   *-- EoM: FDoM

   FUNCTION FDoY( dArg )
   *-------------------------------------------------------------------
   *-- Programmer..: Kenneth Chan
   *-- Date........: 01/05/1993
   *-- Notes.......: Returns January 1 of the year of the date argument
   *--               passed to it.
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 01/05/1993 -- Original Release
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.FDoY(<dArg>))
   *-- Example.....: ? _app.Date.FDoY(date())
   *-- Returns.....: January 1 of the year in dArg
   *-- Parameters..: dArg = date data
   *-------------------------------------------------------------------

      local dJan

      dJan = dArg - day( dArg ) + 1 - 28 * ( month( dArg ) - 1)

   RETURN dJan - day( dJan ) + 1
   *-- EoM: FDoY()

   FUNCTION FDoNextMonth( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Todd Barry  (Originally: FRSTNXTMTH())
   *-- Date........: 08/02/1993
   *-- Notes.......: Returns first day of next month
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 04/04/1992 - Original Release
   *--               02/25/1993 - Jay Parsons - shortened
   *--               08/02/1993 - Jay Parsons - conformed to use FDoM()
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: FDoM()
   *-- Usage.......: DateEx.FDoNextMonth(<dDate>)
   *-- Example.....: ? _app.Date.FDoNextMonth( date() )
   *-- Returns.....: dBASE Date
   *-- Parameters..: dDate  -- date to work from ...
   *-------------------------------------------------------------------

   RETURN ( CLASS::FDoM( (dDate) - day(dDate) + 45 ) )
   *-- EoM: FDoNextMonth()

   FUNCTION FiscalYear
   *-------------------------------------------------------------------
   *-- Programmer..: David Kanter
   *-- Date........: 12/23/1996
   *-- Notes.......: Returns the fiscal year for a given date.  Defaults
   *--               to current date and 10 as the beginning of the
   *--               fiscal year
   *-- Written for.: Visual dBase 5.5
   *-- Rev. History: 12/23/1996 - original function.
   *-- Calls.......: Nothing
   *-- Usage.......: FiscalYear( dDate, nBeginMonth )
   *-- Example.....: ? _app.Date.FiscalYear( {12/01/96}, 10 )   && 1997
   *--               ? _app.Date.FiscalYear( {09/01/96}, 10 )   && 1996
   *--               ? _app.Date.FiscalYear( {09/01/96} )       && 1996
   *-- Returns.....: Numeric fiscal year with century
   *-- Parameters..: dDate  = Date being tested for (date())
   *--               nBegMonth = beginning month of the fiscal year (10)
   *-------------------------------------------------------------------
      parameters dDate, nBegMonth
      local nYear, nMonth
      
      *-- make parameters correct
      dDate = iif( pcount() = 0, date(), dDate )
      if type( "dDate" ) <> "D"
         dDate = ctod( dDate )
      endif
      
      if pCount() < 2
         nBegMonth = 10
      elseif pcount() = 2 .and. type( "nBegMonth" ) <> "N"
         nBegMonth = 10
      endif
      
      nYear = year( dDate )
      nMonth = month( dDate )
      
      if nMonth >= nBegMonth
         nYear = nYear + 1
      endif

   RETURN nYear
   *-- EoM: FiscalYear

   FUNCTION IntlCDOW( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Romain Strieff (CIS: 73612,2257)
   *-- Date........: 09/26/1994
   *-- Notes.......: Function that returns French or German CDOW
   *--               Written on a proposition by Angus Scott Fleming
   *-- Written for.: dBASE IV, 2.0
   *-- Rev. History: 05/17/1994 -- Original Release
   *--               09/26/1994 -- Jay Parsons, shortened
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.IntlCDOW(dDate)
   *-- Example.....: myday = _app.Date.IntlCDOW(date())
   *-- Returns.....: The character day of the week in English/German/
   *--               French
   *-- Parameters..: dDate = a date
   *-------------------------------------------------------------------

      local cDayNames

      * error trap
      if type("dDate") <> "D"
         store date() to dDate
      endif && type("dDate") <> "D"

      do case
         case set("date") $ "AMERICAN,BRITISH,ANSI,USA,MDY,DMY,YMD"
             RETURN cdow(dDate)
                       && return cdow() if not french or german date set
         case set("date") = "FRENCH"
            * Ruler   ->01234567890123456789012345678901234567890
            cDaynames = "         dimanche  lundi     mardi     " + ;
                       "mercredi  jeudi     vendredi  samedi    "
         case set("date") = "GERMAN"
            cDaynames = "         Sonntag   Montag    Dienstag  " + ;
                       "Mittwoch  DonnerstagFreitag   Samstag   "
      endcase

   RETURN trim( substr( cDaynames, 10 * dow(dDate), 10 ) )
   *-- EoM: IntlCDoW()

   FUNCTION IntlCMON( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Romain Strieff (CIS: 73612,2257)
   *-- Date........: 11/03/1996
   *-- Notes.......: Function that returns French or German CMONTH
   *--               Written on a proposition by Angus Scott Fleming
   *--                 
   *--               Romain Strieff wrote this with proper accents in 
   *--               the French fevrier and aout and the German Marz.
   *--               These have been removed because including them 
   *--               with the standard dBASE 5 for Windows font and US 
   *--               language driver causes the accented vowels to be 
   *--               replaced by commas. If using a different font or 
   *--               language driver, replace the accents.		
   *-- Written for.: dBASE IV, 2.0
   *-- Rev History.: 05/17/1994 -- Original
   *--               09/26/1994 -- Modified by Jay Parsons -- see note
   *--               above.
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.IntlCMON(dDate)
   *-- Example.....: mymonth = _app.Date.IntlCMON(date())
   *-- Returns.....: The character month in English/German/French
   *-- Parameters..: dDate a date variable/field
   *-------------------------------------------------------------------

      local cMonths

      * error trap
      if type("dDate") <> "D"
         store date() to dDate
      endif && type("dDate") <> "D"
      
      do case
         case set("date") $ "AMERICAN,BRITISH,ANSI,USA,MDY,DMY,YMD"
              RETURN cmonth(dDate)
                 && RETURN cmonth() if not french or german date set
         case set("date") = "FRENCH"
         * Ruler ->01234567890123456789012345678901234567890
         cMonths = "         janvier   fevrier   mars      " + ;
                   "avril     mai       juin      juillet   " + ;
                   "aout      septembre octobre   novembre  decembre  "
         case set("date") = "GERMAN"
         cMonths = "         Januar    Februar   Marz      " + ;
                   "April     Mai       Juni      Juli      " + ;
                   "August    September Oktober   November  Dezember  "
         endcase

   RETURN trim( substr( cMonths, 10 * month(dDate), 10 ) )
   *-- EoM: IntlCMon()

   FUNCTION IsLeap( nYear )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 08/03/1993
   *-- Notes.......: Is the year given a Leap Year? Year given must be
   *--               after 1500 C.E.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/08/1991 - Jay Parsons - original function
   *--               01/13/1992 - Jay Parsons - updated to handle two-
   *--               digit OR four-digit year.
   *--               08/03/1993 - Jay Parsons - removed change to 
   *--               parameter
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.IsLeap(<nYear>)
   *-- Example.....: ? _app.Date.IsLeap(91)
   *-- Returns.....: Logical (.t./.f.) 
   *-- Parameters..: nYear  = Numeric form of year -- last two digits
   *--                        (i.e., 91), or all four digits 
   *--                        (i.e., 1991)
   *-------------------------------------------------------------------
   
      local nYr, lReturn
   
      *-- deal with two digit year ...
      nYr = nYear
      if nYr < 100
         nYr = nYr + 100 * int( year( date() ) / 100 )
      endif
   
      lReturn = mod( iif( mod( nYr, 100 ) = 0, ;
                      nYr / 100, nYr ), 4 ) = 0
      
   RETURN lReturn
   *-- EoM: IsLeap()

   FUNCTION Jul2Dat( nJulian )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 09/25/1994
   *-- Notes.......: Converts Julian # of days to dBASE Date
   *-- Rev. History: 03/01/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Written for.: dBASE IV
   *-- Rev. History: None
   *-- Calls.......: None
   *-- Called by...: Any
   *-- Usage.......: DateEx.Jul2Dat(nJulian)
   *-- Example.....: ? _app.Date.Jul2Dat(2448691)
   *-- Returns.....: Date
   *-- Parameters..: nJulian = Julian date to convert to dBase Date
   *-------------------------------------------------------------------

   RETURN ( ctod( "01/01/01" ) + (nJulian) - 2415386)
   *-- EoM: Jul2Dat()

   FUNCTION LDoD( dDate, cDay )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 08/02/1993
   *-- Notes.......: Last Date of Day. This function works to give the
   *--               date of the last occurrence in the month of the 
   *--               given given day of the week (i.e., last Monday of 
   *--               the month). It returns a blank date if the day of 
   *--               week is invalid, but is not case sensitive.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 02/25/1992 - Original Release
   *--               08/02/1993 - Jay Parsons - declared nDay private.
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: DateEx.LDoM()
   *--               DateEx.NDoW()
   *-- Usage.......: DateEx.LDoD(<dDate>,"<cDay>")
   *-- Example.....: ? _app.Date.LDoD(DATE(),"Tuesday")
   *-- Returns.....: dBASE Date
   *-- Parameters..: dDate  -- date to work from ...
   *--               cDay   -- Day of week to look for ...
   *-----------------------------------------------------------------------

      local dReturn, nDay
      
      nDay = CLASS::NDoW( cDay )
      dReturn = CLASS::LDoM( dDate )
         
   RETURN dReturn - mod( dow( dReturn ) + 7 - nDay, 7 )
   *-- EoM: LDoD()

   FUNCTION LDoM( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Kenneth Chan 
   *-- Date........: 02/26/1992
   *-- Notes.......: Last Date of Month. Returns the date of the last
   *--               day of the month of the given date.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 02/26/1992 -- Original Release
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.LDoM(<dDate>)
   *-- Example.....: ? _app.Date.LDoM(DATE())
   *-- Returns.....: dBASE Date
   *-- Parameters..: dDate  -- date to work from ...
   *-------------------------------------------------------------------

      local dNxtMonth
   
      dNxtMonth = dDate - day( dDate ) + 45 && middle of next month
   
   RETURN dNxtMonth - day( dNxtMonth )
   *-- EoM: LDoM()

   FUNCTION LDoY( dArg )
   *-------------------------------------------------------------------
   *-- Programmer..: Kenneth Chan
   *-- Date........: 01/05/1993
   *-- Notes.......: Returns December 31 of year in date argument 
   *--               passed to function.
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 01/05/1993 -- Original Release
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: DateEx.LDoM() 
   *-- Usage.......: DateEx.LDoY(<dArg>)
   *-- Example.....: ? _app.Date.LDoY(Date())
   *-- Returns.....: Last Day of Year
   *-- Parameters..: dArg = Date 
   *-------------------------------------------------------------------
      
      local dDec
      
      dDec = dArg - day( dArg ) + 28 * ( 13 - month( dArg ))

   RETURN CLASS::LDoM( dDec )
   *-- EoM: LDoY()

   FUNCTION NDoW( cDay )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302) 
   *-- Date........: 09/25/1994
   *-- Notes.......: Numeric Day of Week -- returns the numeric dow 
   *--               value of a named day of the week for use by some 
   *--               of the other date routines in the library.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 02/25/1992 - original function.
   *--               04/22/1992 - Jay Parsons - modified example and
   *--               descriptions, added ltrim() of argument.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: NDoW(<cDay>)
   *-- Example.....: nDay = oMyString.NDoW("Tues")
   *-- Returns.....: Numeric dow value of day of week given
   *-- Parameters..: cDay  -- Character memvar containing "day" of week
   *--                        ('MONDAY', etc ...)
   *-------------------------------------------------------------------

   RETURN ( at( upper( left( ltrim( cDay ), 3 ) ), ;
          "   SUN MON TUE WED THU FRI SAT" ) / 4 )
   *-- EoD: NDoW

   FUNCTION Num2Dat( nYyMmDd )
   *-------------------------------------------------------------------
   *-- Programmer..: Raymond S. Leventhal (CIS: 71544,2140)
   *-- Date........: 04/22/1994
   *-- Notes.......: Changes Date format from *numeric* yymmdd to dBASE 
   *--               date format. The author uses an AS/400 and 
   *--               downloads data. The YYMMDD format is the date 
   *--               format used by the AS/400.
   *--
   *--               As written, this function will not work for dates 
   *--               in the first decade of a century, such as January 
   *--               1, 2000, because the numeric representation of the 
   *--               date will not have six digits. This appears to be 
   *--               an AS/400 problem. To convert the function to make 
   *--               sense of a date like 101 meaning January 1, 2000,
   *--               use int( nYyMmDd/10000 ) as the year and the 
   *--               result of int( mod( nYyMmDd, 10000 ) / 100 ) as 
   *--               the month.
   *-- Rev. History: 06/04/1992 -- Original version
   *--               04/22/1994 -- Jay Parsons - made independent of 
   *--                             date set() setting
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.Num2Dat(<nYyMmDd>)
   *-- Example.....: ? oMyString.Num2Dat(930820)
   *-- Returns.....: Date format value of nYyMmDd (i.e. 08/20/93)
   *-- Parameters..: nYyMmDd = Numeric value YyMmDd to be converted to 
   *--                         date
   *-------------------------------------------------------------------
      
      local cDateSet, dDate
      
      cDateSet = set("DATE")
      set date AMERICAN
      dDate = ctod(substr(ltrim(str(nYyMmDd)),3,2)+"/"+;
              right(ltrim(str(nYyMmDd)),2)+"/"+;
              left(ltrim(str(nYyMmDd)),2))
      set date &cDateSet.

   RETURN dDate
   *-- EoM: Num2Dat()

   FUNCTION NumDoD( dDate, nWeek, cDay )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 02/24/1992
   *-- Notes.......: Numbered Date of a Day. This function will return 
   *--               the occurrence numbered by its second argument of 
   *--               the day of the week given as its third argument 
   *--               within the month in which the date given as the 
   *--               first argument occurs.
   *--               Example: what if you need the third Monday of the
   *--               month?
   *-                Send to this function a date (any date) of the 
   *--               month, the number you need (first, second...) and 
   *--               the day you need. The function is not case 
   *--               specific.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 02/24/1992 -- Original Release
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: DateEx.FDoD()
   *-- Usage.......: DateEx.NumDoD(<dDate>,<nWeek>,<cDay>)
   *-- Example.....: ? _app.Date.NumDoD({02/03/92},3,"Monday")
   *-- Returns.....: Date
   *-- Parameters..: dDate  =  Any date of the month (and year) needed
   *--               nWeek  =  Number of week you need (i.e., third 
   *--                         cDay of the month
   *--               cDay   =  Character name of day ("Monday", etc.)
   *-------------------------------------------------------------------
      
      local dReturn
      
      dReturn = CLASS::FDoD( dDate, cDay ) + 7 * ( nWeek - 1 )
      
   RETURN dReturn
   *-- EoM: NumDoD()

   FUNCTION Quarter( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Bowen Moursund (CIS: 72662,436)
   *-- Date........: 09/25/1994
   *-- Notes.......: Returns the quarter of the year of a specific 
   *--               date ...
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 02/03/1992 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.Quarter(<dDate>)
   *-- Example.....: ? _app.Date.Quarter({05/25/1992})
   *-- Returns.....: Numeric (integer) value from 1 to 4 (or 0 on 
   *--                   error )
   *-- Parameters..: dDate = date to be checked
   *-------------------------------------------------------------------
      dt = dDate && added, because parameter dDate doesn't register
                 && with the type() function as being defined

   RETURN ( iif( type("dt")="D", ;
            int( ceiling(month(dt)/3) ), 0) )
   *-- EoM: Quarter()

   FUNCTION StoD( cString )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 04/22/1994
   *-- Notes.......: Convert string YYYYMMDD or YYMMDD to a date 
   *--               regardless of SET DATE.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/10/1991 - Original Release
   *--               08/02/1993 - Jay Parsons - cS added to eliminate
   *--               changes to parameter
   *--               04/22/1994 - Jay Parsons - greatly simplified to 
   *--               use Romain Strieff's technique of changing the 
   *--               date setting.
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: DateEx.StoD(<cString>)
   *-- Example.....: ? _app.Date.StoD("19910101")
   *-- Returns.....: Date
   *-- Parameters..: <cString> = Date string you wish converted to 
   *--                           "normal" dBASE date. Must be in either 
   *--                           YYYYMMDD or YYMMDD format.
   *-------------------------------------------------------------------
      
      local cS, cDateSet, dReturn
      
      cS = cString
      if len( cS ) < 8
         cS = left( str( year( date() ), 4 ), 2 ) + cS
      endif
      cDateSet = set("DATE")
      set date AMERICAN
      dReturn = ctod( substr( cS, 5, 2 ) + "/" + right( cS, 2 ) + "/" ;
                + left( cS, 4 ) )
      set date &cDateSet.
      
   RETURN dReturn
   *-- EoM: StoD()

   FUNCTION Text2Date( cDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Angus Scott-Fleming [75500,3223]
   *-- Date........: 04/05/1994
   *-- Notes.......: Convert string from dd-mmm-yy to date
   *-- Written for.: dBASE IV, 1.5+
   *-- Rev. History: 04/05/1994  first pass
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: dateex.Text2Date(<string>)
   *-- Example.....: ? _app.Date.Text2Date("02-Feb-94")
   *-- Returns.....: date, or null date for bad input
   *-- Parameters..: cDate = string to be converted
   *-------------------------------------------------------------------
      
      local nFirstDash, nLastDash, cDay, cMonth, cYear, nDay, ;
            nMonth, nYear
       
      *-- assumes date is of form 0d-Mmm-yy or 0d-Mmm-yyyy
      *-- check input for two hyphens
      nFirstDash = at("-",cDate)
      nLastDash = at("-",cDate,2)
      if nFirstDash = 0 .or. nLastDash = 0
         * bogus input - return an empty date
         return {}
      endif
      * extract characters for day, month, and year
      cDay = left(cDate,nFirstDash - 1)
      cMonth = substr(cDate, nFirstDash + 1, ;
               nLastDash - nFirstDash - 1)
      cYear  = substr(cDate, nLastDash + 1)
      nDay   = val(cDay)
      * note: this next line should be expanded to deal with things 
      * like "Sept"
      nMonth = ( at(upper(cMonth), ;
                "__,JAN,FEB,MAR,APR,MAY,JUN,JUL,AUG,SEP,OCT,NOV,DEC") );
                / 4
      nYear  = val(cYear)
      * validate numbers
      do case
         case nDay = 0 .or. nMonth = 0 .or. nYear = 0
              * bogus input - return an empty date
              return {}
         case nDay > 31 .or. nMonth > 12
              * bogus input - return an empty date
              * could expand this checking to check for Feb 30 and so 
              * forth
              return {}
         otherwise
              * check year for century
              do case
                 case nYear < 100 .and. year(date()) < 2000
                      store 1900 + nYear to nYear
                 case nYear < 100 .and. year(date()) > 1999
                      store 2000 + nYear to nYear
              endcase
      endcase
      
   RETURN ctod(str(nMonth,2,0) + "/" + str(nDay,2,0) + "/" + ;
          str(nYear,4,0) )
   *-- EoM: Text2Date()

   FUNCTION WeekNo( dDate, nFDoW )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 70160,340)
   *-- Date........: 05/26/1994
   *-- Notes.......: Returns the week number of the year of a date 
   *--               (from beginning of the year). By default, week #1 
   *--               begins on the first Sunday of the year. Days in 
   *--               any partial week before that date are assigned to 
   *--               week #0. To use this function but start week # 1 
   *--               on a different day, use the second parameter to 
   *--               specify the day of the week the first occurrence 
   *--               in the year of which starts week #1, 2 for 
   *--               Monday through 7 for Saturday.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/10/1991 - original function.
   *--               11/14/1991 - Ken Mayer - expanded for readability 
   *--               05/26/1994 - Frank A. Deviney, Jr. - added second 
   *--                            parameter
   *--               11/02/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: DateEx.DoY()
   *-- Usage.......: DateEx.WeekNo(<dDate>,<nFDoW>)
   *-- Example.....: ? _app.Date.WeekNo({01/01/91})   = 0
   *--               ? _app.Date.Weekno({01/01/91},3) = 1
   *--               ? _app.Date.Weekno({01/08/91})   = 1
   *--               ? _app.Date.Weekno({01/08/91},dow(FDoY({01/08/91)))= 2
   *-- Returns.....: Numeric value of week number
   *-- Parameters..: dDate  = Date being tested for ...
   *--               nFDoW  = first day of week [1..7]
   *-----------------------------------------------------------------------
      
      local dBaseDate, nFD
      
      nFD = iif (pcount() < 2, 1, nFDoW )
      dBaseDate = dDate - CLASS::DoY(dDate)
      dBaseDate = dBaseDate - mod( dow( dBaseDate - nFD ), 7 )

   RETURN int( (dDate - dBaseDate) / 7 )
   *-- EoM: WeekNo()

ENDCLASS
*-- End of Class: DateEx

*-----------------------------------------------------------------------
*-- End of File: DATEEX.CC
*-----------------------------------------------------------------------