*-- File: PROGRESS.CC

*----------------------------------------------------------------------
*-- Description:
*
*      This UI file is designed to give a progress bar, which can be
*      placed on a form and updated as some long progress is executed.
*
*-- Programmers: 
*
*      Alan Katz
*
*-- History: 
*
*      ????       -- original
*      12/01/1996 -- Modified just enough for the dUFLP library
*                    (Added ProgBarColor property -- KJM)
*
*-- Instantiation: 
*
*      set procedure to PROGRESS.cc additive
*         In the form's 'OnOpen' set custom properties (or when 
*         instantiating the form ...):
*             this.ProgBar1.nMax         = 100
*             this.ProgBar1.Interval     = 1
*             this.ProgBar1.ProgBarColor = "g/g"
*             this.ProgBar1.Setup(this.ProgBar1.nMax)
*                  OR
*          Assuming code that instantiates the form in your program,
*          you might do something like:
*
*            set procedure to myprogform.wfm
*            fProgForm = new MyProgFormForm()
*            fProgForm.MDI = .f.
*            fProgForm.ProgBar1.nMax         = reccount() 
*                                               && number of records 
*                                               && in table
*            fProgForm.ProgBar1.Interval     = 1
*            fProgForm.ProgBar1.ProgBarColor = "rb/rb" && purple
*            fProgForm.ProgBar1.Setup (reccount())
*            fProgForm.Open()
*            nCount = 0
*            scan
*               nCount = nCount + 1
*               fProgForm.ProgBar1.Update( nCount )
*               *-- do something
*            endscan
*
*-- Properties:
*
*--  nMax 
*      Maximum number of iterations (default is 0 -- you must set this
*      to a useable value)
*
*--  Interval
*      When to update the progress bar (default is 1, as in
*      every time through a loop ...)
*
*--  ProgBarColor
*      Progress bar color -- note that the color should be set to the 
*      same for foreground and background. Default is "r/r" (red)
*
*-- Methods:
*
*--  Setup(<nMax>)
*      Instantiates progress bar, sets maximum value
*
*--  Update(<nNumber>)
*      Updates progress bar, nNumber is current number processed
*
*--  Reset()
*      Resets progress bar back to beginning (left). Does not reset
*      maximum value or other values used ...
*----------------------------------------------------------------------

CLASS ProgBar(f,n) OF RECTANGLE(f,n) Custom

   this.Height      = 1.06
   this.Width       = 10
   this.BorderStyle = 2

   *-- Custom properties:
   this.nMax         = 0
   this.Interval     = 1
   this.ProgBarColor = "r/r"

   PROCEDURE Setup(nMaximum)
      this.nMax = nMaximum
      if this.nMax <= 100
         this.nInterval = 1
         elseif this.nMax <= 1000
            this.nInterval = 5
         elseif this.nMax <= 10000
            this.nInterval = 20
      else
         this.nInterval = 50
      endif
   
      *-- if the rectangle used to display the actual 
      *-- progress bar is not defined, define it
      if type('This.Parent.progdisp') # 'O'
   
         DEFINE RECTANGLE PROGDISP OF This.Parent;
            Property;
              height      this.height-.12,;
              width       .17,;
              left        this.left+.17,;
              Top         this.top+.06,;
              Pageno      this.Pageno,;
              ColorNormal this.ProgBarColor,;
              BorderStyle 0
      endif
   
   PROCEDURE UPDATE(nNum)
      this.nCurrNum = nNum
      this.TestNum = round(this.nCurrnum/this.nInterval,3)

      if this.testnum == int(this.testnum)
         this.Parent.Progdisp.visible = .t.
         local nDecimalDone,nWidth
         nDecimalDone = round(this.nCurrNum/this.nMax,2)
         nWidth = this.width-.35
         if (nDecimaldone*nWidth) > (This.width-.35)
            this.Parent.Progdisp.Width = (This.Width-.35)
         else
            this.Parent.Progdisp.Width = (nDecimalDone*nWidth)
         endif
      endif

   PROCEDURE RESET
      this.Parent.Progdisp.Width = .17

ENDCLASS
*-- End of Class: ProgBar

*-----------------------------------------------------------------------
*-- End of File: PROGRESS.CC
*-----------------------------------------------------------------------